@@ -136,6 +136,8 @@ class TestApp(GoodAppTestCase):
             '--restrict-file-names', 'windows,lower',
             '--quota', '10m',
             '--max-filename-length', '100',
+            '--user-agent', 'ΑΒΓαβγ',
+            '--remote-encoding', 'latin1',
         ])
         with cd_tempdir():
             builder = Builder(args)
@@ -160,7 +162,7 @@ class TestApp(GoodAppTestCase):
         with tempfile.NamedTemporaryFile() as in_file:
             in_file.write(self.get_url('/').encode('utf-8'))
             in_file.write(b'\n')
-            in_file.write(self.get_url('/blog/').encode('utf-8'))
+            in_file.write(self.get_url('/blog/?ðfßðfëéå').encode('utf-8'))
             in_file.flush()
 
             args = arg_parser.parse_args([
@@ -260,6 +262,8 @@ class TestApp(GoodAppTestCase):
             ('--warc-file=test', '--timestamping'),
             ('--warc-file=test', '--continue'),
             ('--lua-script=blah.lua', '--python-script=blah.py'),
+            ('--no-iri', '--local-encoding=shiftjis'),
+            ('--no-iri', '--remote-encoding=shiftjis'),
         ]
 
         for arg_item in arg_items:
@@ -649,6 +653,81 @@ class TestApp(GoodAppTestCase):
         self.assertEqual(0, exit_code)
         self.assertGreaterEqual(4, builder.factory['Statistics'].files)
 
+    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    def test_local_encoding(self):
+        arg_parser = AppArgumentParser()
+
+        with tempfile.NamedTemporaryFile() as in_file:
+            in_file.write(self.get_url('/?qwerty').encode('utf-32-le'))
+            in_file.write('\n'.encode('utf-32-le'))
+            in_file.flush()
+
+            opts = [
+                self.get_url('/?asdf'),
+                '--local-encoding', 'utf-32-le',
+                '--input-file', in_file.name
+            ]
+
+            opts = [string.encode('utf-32-le') for string in opts]
+
+            args = arg_parser.parse_args(opts)
+            builder = Builder(args)
+
+            with cd_tempdir():
+                engine = builder.build()
+                exit_code = yield engine()
+
+        self.assertEqual(0, exit_code)
+        self.assertEqual(2, builder.factory['Statistics'].files)
+
+    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    def test_no_iri(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/'),
+            '--no-iri',
+            '--no-robots'
+        ])
+        builder = Builder(args)
+
+        with cd_tempdir():
+            engine = builder.build()
+            exit_code = yield engine()
+
+        self.assertEqual(0, exit_code)
+        self.assertEqual(1, builder.factory['Statistics'].files)
+
+
+class SimpleHandler(tornado.web.RequestHandler):
+    def get(self):
+        self.write(b'OK')
+
+
+class TestAppHTTPS(tornado.testing.AsyncHTTPSTestCase):
+    def get_app(self):
+        return tornado.web.Application([
+            (r'/', SimpleHandler)
+        ])
+
+    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    def test_https_only(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/?1'),
+            self.get_url('/?2').replace('https://', 'http://'),
+            '--https-only',
+            '--no-robots',
+            '--no-check-certificate',
+        ])
+        builder = Builder(args)
+
+        with cd_tempdir():
+            engine = builder.build()
+            exit_code = yield engine()
+
+        self.assertEqual(0, exit_code)
+        self.assertEqual(1, builder.factory['Statistics'].files)
+
 
 class TestAppBad(BadAppTestCase):
     def setUp(self):
@@ -671,3 +750,20 @@ class TestAppBad(BadAppTestCase):
         cookies = list(builder.factory['CookieJar'])
         _logger.debug('{0}'.format(cookies))
         self.assertEqual(2, len(cookies))
+
+    @tornado.testing.gen_test(timeout=DEFAULT_TIMEOUT)
+    def test_non_http_redirect(self):
+        arg_parser = AppArgumentParser()
+        args = arg_parser.parse_args([
+            self.get_url('/non_http_redirect'),
+            '--recursive',
+            '--no-robots'
+        ])
+        builder = Builder(args)
+
+        with cd_tempdir():
+            engine = builder.build()
+            exit_code = yield engine()
+
+        self.assertEqual(0, exit_code)
+        self.assertEqual(0, builder.factory['Statistics'].files)
