@@ -237,4 +237,22 @@ function resetPage() {
 	setupEvents();
 	applyDefaultPageSettings();
 	page.customHeaders = defaultPageHeaders;
+}
+
+var EVENT_SELECTORS = [ '[onload]', '[onunload]', '[onabortonclick]',
+		'[ondblclick]', '[onmousedown]', '[onmousemove]', '[onmouseout]',
+		'[onmouseover]', '[onmouseup]', '[onkeydown]', '[onkeypress]',
+		'[onkeyup]' ].join(' ');
+
+// Return whether the page has script elements or HTML event attributes
+function isPageDynamic() {
+	var result = page.evaluate(function() {
+		return document.getElementsByTagName('script').length
+				|| document.querySelector(EVENT_SELECTORS);
+	});
+
+	if (result) {
+		return true;
+	}
+
 }
\ No newline at end of file
