@@ -336,8 +336,11 @@ class WebProcessorSession(object):
         and self._rich_client_session \
         and self._rich_client_session.redirect_tracker \
         and self._rich_client_session.redirect_tracker.is_redirect \
-        and any([isinstance(url_filter, SpanHostsFilter) for url_filter in
-                test_info['failed']]):
+        and len(test_info['failed']) == 1 \
+        and all([
+            isinstance(url_filter, SpanHostsFilter)
+            for url_filter in test_info['failed']
+        ]):
             return True, 'redirect'
 
         else:
@@ -414,7 +417,7 @@ class WebProcessorSession(object):
         return True
 
     @classmethod
-    def _parse_url(cls, url, encoding):
+    def parse_url(cls, url, encoding):
         '''Parse and return a URLInfo.
 
         This function logs a warning if the URL cannot be parsed and returns
@@ -506,7 +509,7 @@ class WebProcessorSession(object):
         linked_url_infos = set()
 
         for url in inline_urls:
-            url_info = self._parse_url(url, encoding)
+            url_info = self.parse_url(url, encoding)
             if url_info:
                 url_record = self._url_item.child_url_record(
                     url_info, inline=True, encoding=encoding
@@ -515,7 +518,7 @@ class WebProcessorSession(object):
                     inline_url_infos.add(url_info)
 
         for url in linked_urls:
-            url_info = self._parse_url(url, encoding)
+            url_info = self.parse_url(url, encoding)
             if url_info:
                 url_record = self._url_item.child_url_record(
                     url_info, encoding=encoding, link_type=link_type
@@ -670,7 +673,8 @@ class WebProcessorSession(object):
 class PhantomJSController(object):
     '''PhantomJS Page Controller.'''
     def __init__(self, client, wait_time=1, num_scrolls=5, snapshot=True,
-    warc_recorder=None, viewport_size=(1024, 3072), paper_size=(2048, 6144)):
+    warc_recorder=None, viewport_size=(1024, 3072), paper_size=(2048, 6144),
+    smart_scroll=True):
         self.client = client
         self._wait_time = wait_time
         self._num_scrolls = num_scrolls
@@ -678,6 +682,7 @@ class PhantomJSController(object):
         self._warc_recorder = warc_recorder
         self._viewport_size = viewport_size
         self._paper_size = paper_size
+        self._smart_scroll = smart_scroll
         self._actions = []
         self._action_warc_record = None
 
@@ -700,15 +705,21 @@ class PhantomJSController(object):
     @tornado.gen.coroutine
     def control(self, remote):
         '''Scroll the page.'''
+        num_scrolls = self._num_scrolls
+
+        if self._smart_scroll:
+            is_page_dynamic = yield remote.call('isPageDynamic')
+
+            if not is_page_dynamic:
+                num_scrolls = 0
+
         url = yield remote.eval('page.url')
         total_scroll_count = 0
 
-        self._log_action('wait', self._wait_time)
-        yield wpull.util.sleep(self._wait_time)
-
-        for scroll_count in range(self._num_scrolls):
+        for scroll_count in range(num_scrolls):
             _logger.debug('Scrolling page. Count={0}.'.format(scroll_count))
 
+            pre_scroll_counter_values = remote.resource_counter.values()
             scroll_position = yield remote.eval('page.scrollPosition')
             scroll_position['top'] += self._viewport_size[1]
 
@@ -718,10 +729,21 @@ class PhantomJSController(object):
             yield remote.call('page.sendEvent', 'keydown', 'PageDown')
             yield remote.call('page.sendEvent', 'keyup', 'PageDown')
 
+            total_scroll_count += 1
+
             self._log_action('wait', self._wait_time)
             yield wpull.util.sleep(self._wait_time)
 
-            total_scroll_count += 1
+            if remote.resource_counter.values() == pre_scroll_counter_values \
+            and self._smart_scroll:
+                break
+
+        for dummy in range(remote.resource_counter.pending):
+            if remote.resource_counter.pending:
+                self._log_action('wait', self._wait_time)
+                yield wpull.util.sleep(self._wait_time)
+            else:
+                break
 
         _logger.info(
             gettext.ngettext(
